@@ -37,7 +37,7 @@ THE SOFTWARE. */
 
 #include "stdafx.hpp"
 #include "Preferences.hpp"
-#include "OptionsDialog.hpp"
+#include "dialogs/OptionsDialog.hpp"
 #include "ThereminView.hpp"
 #include "File.hpp"
 #include "FSFile.hpp"
@@ -52,7 +52,6 @@ using namespace ting;
 
 
 ThereminView::ThereminView() :
-		optionsDialog(MainWindow::Inst()),
 		tappedX(0)
 {
 	this->add_events(
@@ -89,71 +88,38 @@ ThereminView::ThereminView() :
 		);
 
 	//set default timbre
-	this->ApplyTimbre(Preferences::Inst().GetTimbre());
+	this->instr.ApplyTimbre(Preferences::Inst().GetTimbre());
 }
 
 
 
 void ThereminView::ShowOptionsDialog(){
 //	TRACE(<< "ThereminView::ShowOptionsDialog(): enter" << std::endl)
-	this->optionsDialog.SetNumOctaves(Preferences::Inst().GetNumOctaves());
-	this->optionsDialog.SetTimbre(Preferences::Inst().GetTimbre());
-	this->optionsDialog.SetLockToTappedOctave(Preferences::Inst().GetLockCurrentOctave());
+	OptionsDialog d(MainWindow::Inst());
+	d.SetNumOctaves(Preferences::Inst().GetNumOctaves());
+	d.SetTimbre(Preferences::Inst().GetTimbre());
+	d.SetLockToTappedOctave(Preferences::Inst().GetLockCurrentOctave());
 
-	switch(this->optionsDialog.run()){
+	switch(d.run()){
 		case Gtk::RESPONSE_OK: //OK
-			Preferences::Inst().SetNumOctaves(this->optionsDialog.GetNumOctaves());
-			Preferences::Inst().SetTimbre(this->optionsDialog.GetTimbre());
-			Preferences::Inst().SetLockCurrentOctave(this->optionsDialog.GetLockToTappedOctave());
-			this->ApplyTimbre(Preferences::Inst().GetTimbre());
+			Preferences::Inst().SetNumOctaves(d.GetNumOctaves());
+			Preferences::Inst().SetTimbre(d.GetTimbre());
+			Preferences::Inst().SetLockCurrentOctave(d.GetLockToTappedOctave());
+			this->instr.ApplyTimbre(Preferences::Inst().GetTimbre());
 
 			Preferences::Inst().SaveToFile();
 			break;
 		default: //Cancel and all other cases
-		case Gtk::RESPONSE_CANCEL:
-			this->optionsDialog.SetNumOctaves(Preferences::Inst().GetNumOctaves());
-			this->optionsDialog.SetTimbre(Preferences::Inst().GetTimbre());
-			this->optionsDialog.SetLockToTappedOctave(Preferences::Inst().GetLockCurrentOctave());
 			break;
 	}
 
-	this->optionsDialog.hide();
+	d.hide();
 	MainWindow::Inst().queue_draw();//request redrawing entire window
 //	TRACE(<< "ThereminView::ShowOptionsDialog(): exit" << std::endl)
 }
 
 
 
-void ThereminView::ApplyTimbre(Preferences::E_Timbre timbre){
-	switch(timbre){
-		default:
-		case Preferences::SINE_PLUS_ADSR:
-			this->instr = aumiks::SineWaveInstrument::New();
-			break;
-		case Preferences::PLUCKED_STRING:
-			this->instr = aumiks::PluckedStringInstrument::New();
-			break;
-		case Preferences::TROMBONE:
-			this->instr = aumiks::TromboneInstrument::New();
-			break;
-		case Preferences::CLARINET:
-			this->instr = aumiks::ClarinetInstrument::New();
-			break;
-	}
-}
-
-
-
-Preferences::E_Timbre ThereminView::GetCurrentTimbreType()const{
-	if(this->instr.DynamicCast<aumiks::PluckedStringInstrument>().IsValid()){
-		return Preferences::PLUCKED_STRING;
-	}else{
-		return Preferences::SINE_PLUS_ADSR;
-	}
-}
-
-
-
 bool ThereminView::OnExposeEvent(GdkEventExpose* event){
 //	TRACE(<< "ThereminView::OnExposeEvent(): enter" << std::endl)
 
@@ -174,7 +140,6 @@ bool ThereminView::OnExposeEvent(GdkEventExpose* event){
 		c->clip();
 	}
 
-
 	c->set_line_width(1.0);
 	c->set_source_rgb(0, 0, 0);//black
 	c->select_font_face("Sans", Cairo::FONT_SLANT_NORMAL, Cairo::FONT_WEIGHT_NORMAL);
@@ -218,39 +183,63 @@ void ThereminView::DrawOctave(
 		Cairo::RefPtr<Cairo::Context> c,
 		ting::uint w,
 		ting::uint h,
-		ting::uint basePitch)
+		ting::uint basePitch
+	)
 {
 	float step = float(w) / 12.0f;
 	float curX = 0;
 	float height = float(h);
-	c->set_source_rgb(0, 0, 0);//black
+
+	//draw white background
+	c->set_source_rgb(1, 1, 1);//white
+	c->rectangle(0, 0, w, h);
+	c->fill();
 
 	for(ting::uint i = 0, n = basePitch % 12; i < 12; ++i, ++n){
 		if(n >= 12)
 			n -= 12;
 
-		c->move_to(curX, 0);
-		c->line_to(curX, height);
-		c->stroke();
-
 		ASSERT(n < 12)
-		//draw notes
+		//draw notes and black bars for sharp notes
 		if(notes[n]){
+			//draw note
 			Cairo::TextExtents extents;
 			c->get_text_extents(notes[n], extents);
+			c->set_source_rgb(0, 0, 0);//black
 			c->move_to(curX + step / 2 - (extents.width / 2) - extents.x_bearing, height / 2);
 			c->show_text(notes[n]);
+		}else{
+			//draw black bar (sharp note)
+			c->set_source_rgb(0.7, 0.7, 0.7);//grey
+			c->rectangle(curX, 0, step, height);
+			c->fill();
 		}
 
+		c->set_source_rgb(0, 0, 0);//black
+		c->move_to(curX, 0);
+		c->line_to(curX, height);
+
+		c->save();
+		c->set_identity_matrix();
+		c->set_line_width(2);
+		c->stroke();
+		c->restore();
+
 		curX += step;
 	}
+
+	//draw horizontal line delimiting octaves
 	c->move_to(0, h);
 	c->line_to(w, h);
+	c->save();
+	c->set_identity_matrix();
+	c->set_line_width(3);
 	c->stroke();
+	c->restore();
 
 	//draw octave number
 	c->set_source_rgb(0.5f, 0, 0);//dark red
-	c->move_to(0, h);
+	c->move_to(5, h - 5);
 	char buf[33];
 	snprintf(buf, sizeof(buf)-1, "%u", basePitch / 12);
 	buf[32] = 0;
@@ -259,17 +248,6 @@ void ThereminView::DrawOctave(
 
 
 
-struct MenuPlacer{
-	int menuX, menuY;
-	void onPopupMenuPosition(int& x, int& y, bool& pushIn){
-		x = this->menuX;
-		y = this->menuY;
-		pushIn = true;
-	}
-};
-
-
-
 bool ThereminView::OnKeyPress(GdkEventKey* event){
 	TRACE(<<"ThereminView::OnKeyPress(): keyval = " << event->keyval << std::endl)
 	switch(event->keyval){
@@ -364,19 +342,14 @@ ThereminView::FreqVol ThereminView::CalcFreqAndVol(
 	FreqVol ret;
 	x -= width / real(24); //subtract half of halfstep onscreen width
 
-	ting::uint octHeight = height / real(Preferences::Inst().GetNumOctaves());
+	ting::uint octHeight = ting::uint(height / real(Preferences::Inst().GetNumOctaves()));
 
 	ting::ClampBottom(y, real(0));
 	ASSERT(y >= real(0))
 
 	//find the screen octave number of the touched octave
-	ting::uint touchedOctave = 0;
-	for(ting::uint i = 1; i <= Preferences::Inst().GetNumOctaves(); ++i){
-		if(ting::uint(y) < i * octHeight){
-			touchedOctave = i;
-			break;
-		}
-	}
+	ting::uint touchedOctave = ting::uint(y) / octHeight;
+	ting::ClampTop(touchedOctave, Preferences::Inst().GetNumOctaves() - 1);
 
 	if(Preferences::Inst().GetLockCurrentOctave()){
 		//if press event then update current playing octave, this is to prevent volume leaps
@@ -386,17 +359,15 @@ ThereminView::FreqVol ThereminView::CalcFreqAndVol(
 		this->curOctave = touchedOctave;
 	}
 
-	real pow = 1;
-	for(ting::uint j = 0; j < (Preferences::Inst().GetNumOctaves() - this->curOctave); ++j)
-		pow *= 2;
+	ting::uint octavesShift = (Preferences::Inst().GetNumOctaves() - 1 - this->curOctave) * 12;
 
 	ret.freq = CalcFreq(
-			real(Preferences::Inst().GetBaseFreq()) * pow,
-			x / width
+			real(27.5f),
+			x / width + real(Preferences::Inst().GetBasePitch() + octavesShift) / real(12)
 		);
 
 	if(touchedOctave == this->curOctave)
-		ret.vol = ThereminView::CalcVol(y - real(octHeight * (this->curOctave - 1)), octHeight);
+		ret.vol = ThereminView::CalcVol(y - real(octHeight * this->curOctave), octHeight);
 	else if(touchedOctave < this->curOctave)
 		ret.vol = 0xff;//max volume
 	else //touchedOctave > this->curOctave
@@ -422,14 +393,13 @@ bool ThereminView::OnButtonPressEvent(GdkEventButton* event){
 			true //press event
 		);
 
-	if(this->ch.IsValid())
-		this->ch->Stop();
+	this->instr.Stop();
 
 #ifndef M_NO_AUDIO
-	this->ch = instr->CreateChannel();
-	this->ch->SetFrequency(fv.freq);
-	this->ch->SetVolume(fv.vol);
-	this->ch->Play();
+	this->instr.RenewChannel();
+	this->instr.SetFrequency(fv.freq);
+	this->instr.SetVolume(fv.vol);
+	this->instr.Play();
 #endif
 
 	if(Preferences::Inst().GetBroadcast()){
@@ -454,8 +424,7 @@ bool ThereminView::OnButtonReleaseEvent(GdkEventButton* event){
 //	this->queue_draw_area(this->tappedX - 2, 0, this->tappedX + 2, this->get_height());
 	this->tappedX = -1;
 
-	if(this->ch.IsValid())
-		this->ch->Stop();
+	this->instr.Stop();
 
 	if(Preferences::Inst().GetBroadcast()){
 		//send NOTE_OFF music action event to main thread
@@ -475,7 +444,7 @@ bool ThereminView::OnMotionEvent(GdkEventMotion* event){
 		this->tappedX = float(event->x);
 //		this->queue_draw_area (this->tappedX - 2, 0, this->tappedX + 2, this->get_height());
 
-		if(this->ch.IsValid()){
+		if(this->instr.IsChannelValid()){
 			FreqVol fv = this->CalcFreqAndVol(
 					float(this->get_width()),
 					float(this->get_height()),
@@ -484,8 +453,8 @@ bool ThereminView::OnMotionEvent(GdkEventMotion* event){
 					false //not a press event
 				);
 
-			this->ch->SetFrequency(fv.freq);
-			this->ch->SetVolume(fv.vol);
+			this->instr.SetFrequency(fv.freq);
+			this->instr.SetVolume(fv.vol);
 
 			if(Preferences::Inst().GetBroadcast()){
 				//send NOTE_ON music action event to main thread
