@@ -28,8 +28,7 @@ THE SOFTWARE. */
  * Created on February 16, 2009, 10:28 PM
  */
 
-#ifndef M_aumiks_hpp
-#define	M_aumiks_hpp
+#pragma once
 
 #include <ting/Singleton.hpp>
 #include <ting/Exc.hpp>
@@ -152,4 +151,3 @@ public:
 };
 
 } //~namespace
-#endif //~once
